@@ -13,7 +13,6 @@ import {
   DataQueryResponse,
   DataSourceApi,
   dateTimeForTimeZone,
-  Field,
   GrafanaTheme2,
   hasLogsContextSupport,
   hasLogsContextUiSupport,
@@ -29,13 +28,17 @@ import {
   urlUtil,
   LogSortOrderChangeEvent,
   LoadingState,
+  rangeUtil,
 } from '@grafana/data';
-import { convertRawToRange } from '@grafana/data/src/datetime/rangeutil';
+import { Trans } from '@grafana/i18n';
 import { config, getAppEvents } from '@grafana/runtime';
 import { ScrollContainer, usePanelContext, useStyles2 } from '@grafana/ui';
 import { getFieldLinksForExplore } from 'app/features/explore/utils/links';
+import { ControlledLogRows } from 'app/features/logs/components/ControlledLogRows';
 import { InfiniteScroll } from 'app/features/logs/components/InfiniteScroll';
 import { LogRowContextModal } from 'app/features/logs/components/log-context/LogRowContextModal';
+import { LogLineContext } from 'app/features/logs/components/panel/LogLineContext';
+import { LogList } from 'app/features/logs/components/panel/LogList';
 import { PanelDataErrorView } from 'app/features/panel/components/PanelDataErrorView';
 import { combineResponses } from 'app/plugins/datasource/loki/mergeResponses';
 
@@ -45,15 +48,20 @@ import { LogRows } from '../../../features/logs/components/LogRows';
 import { COMMON_LABELS, dataFrameToLogsModel, dedupLogRows } from '../../../features/logs/logsModel';
 
 import {
+  GetFieldLinksFn,
+  isCoreApp,
   isIsFilterLabelActive,
+  isLogLineMenuCustomItems,
   isOnClickFilterLabel,
   isOnClickFilterOutLabel,
   isOnClickFilterOutString,
   isOnClickFilterString,
   isOnClickHideField,
   isOnClickShowField,
+  isOnLogOptionsChange,
   isOnNewLogsReceivedType,
   isReactNodeArray,
+  isSetDisplayedFields,
   onNewLogsReceivedType,
   Options,
 } from './types';
@@ -85,6 +93,9 @@ interface LogsPanelProps extends PanelProps<Options> {
    * Called from the "eye" icon in Log Details to request hiding the displayed field. If ommited, a default implementation is used.
    * onClickHideField?: (key: string) => void;
    *
+   * Called from the new Log Details Panel when fields are reordered. If ommited, a default implementation is used.
+   * setDisplayedFields?: (key: string) => void;
+   *
    * Passed to the LogRowMenuCell component to be rendered before the default actions in the menu.
    * logRowMenuIconsBefore?: ReactNode[];
    *
@@ -93,6 +104,30 @@ interface LogsPanelProps extends PanelProps<Options> {
    *
    * Callback to be invoked when enableInfiniteScrolling and new logs have been received after an scroll event.
    * onNewLogsReceived?: (allLogs: DataFrame[], newLogs: DataFrame[]) => void;
+   *
+   * Log Controls props:
+   *
+   * Enables a sidebar with controls for scrolling, sort order, deduplication, filtering, timestamps, wrapping, etc.
+   * showControls?: boolean
+   *
+   * If controls are enabled, the component will use this key to store changes to the aforementioned options.
+   * controlsStorageKey?: string
+   *
+   * If controls are enabled, this function is called when a change is made in one of the options from the controls.
+   * onLogOptionsChange?: (option: LogListControlOptions, value: string | boolean | string[]) => void;
+   *
+   * When the feature toggle newLogsPanel is enabled, you can pass extra options to the LogLineMenu component.
+   * These options are an array of items with { label, onClick } or { divider: true } for dividers.
+   * logLineMenuCustomItems?: LogLineMenuCustomItem[];
+   *
+   * Use the default, bigger, font size, or a smaller one. Defaults to "default".
+   * fontSize?: 'default' | 'small'
+   *
+   * Set the mode used by the Log Details panel. Displayed as a sidebar, or inline below the log line. Defaults to "inline".
+   * detailsMode?: 'inline' | 'sidebar'
+   *
+   * When showing timestamps, toggle between showing nanoseconds or milliseconds.
+   * timestampResolution?: 'ms' | 'ns'
    */
 }
 interface LogsPermalinkUrlState {
@@ -108,6 +143,8 @@ export const LogsPanel = ({
   timeZone,
   fieldConfig,
   options: {
+    showControls,
+    controlsStorageKey,
     showLabels,
     showTime,
     wrapLogMessage,
@@ -121,13 +158,21 @@ export const LogsPanel = ({
     onClickFilterOutLabel,
     onClickFilterOutString,
     onClickFilterString,
+    onLogOptionsChange,
     isFilterLabelActive,
     logRowMenuIconsBefore,
     logRowMenuIconsAfter,
+    logLineMenuCustomItems,
     enableInfiniteScrolling,
     onNewLogsReceived,
+    fontSize,
+    syntaxHighlighting,
+    detailsMode: detailsModeProp,
+    noInteractions,
+    timestampResolution,
     ...options
   },
+  height,
   id,
 }: LogsPanelProps) => {
   const isAscending = sortOrder === LogsSortOrder.Ascending;
@@ -143,9 +188,9 @@ export const LogsPanel = ({
   const [panelData, setPanelData] = useState(data);
   const dataSourcesMap = useDatasourcesFromTargets(panelData.request?.targets);
   // Prevents the scroll position to change when new data from infinite scrolling is received
-  const keepScrollPositionRef = useRef(false);
+  const keepScrollPositionRef = useRef<null | 'infinite-scroll' | 'user'>(null);
   let closeCallback = useRef<() => void>();
-  const { eventBus, onAddAdHocFilter } = usePanelContext();
+  const { app, eventBus, onAddAdHocFilter } = usePanelContext();
 
   useEffect(() => {
     getAppEvents().publish(
@@ -267,13 +312,19 @@ export const LogsPanel = ({
   // Important to memoize stuff here, as panel rerenders a lot for example when resizing.
   const [logRows, deduplicatedRows, commonLabels] = useMemo(() => {
     const logs = panelData
-      ? dataFrameToLogsModel(panelData.series, panelData.request?.intervalMs, undefined, panelData.request?.targets)
+      ? dataFrameToLogsModel(
+          panelData.series,
+          panelData.request?.intervalMs,
+          undefined,
+          panelData.request?.targets,
+          Boolean(enableInfiniteScrolling)
+        )
       : null;
     const logRows = logs?.rows || [];
     const commonLabels = logs?.meta?.find((m) => m.label === COMMON_LABELS);
     const deduplicatedRows = dedupLogRows(logRows, dedupStrategy);
     return [logRows, deduplicatedRows, commonLabels];
-  }, [dedupStrategy, panelData]);
+  }, [dedupStrategy, enableInfiniteScrolling, panelData]);
 
   const onPermalinkClick = useCallback(
     async (row: LogRowModel) => {
@@ -289,8 +340,12 @@ export const LogsPanel = ({
   }, [data]);
 
   useLayoutEffect(() => {
+    if (config.featureToggles.newLogsPanel) {
+      return;
+    }
     if (!logsContainerRef.current || !scrollElement || keepScrollPositionRef.current) {
-      keepScrollPositionRef.current = false;
+      keepScrollPositionRef.current =
+        keepScrollPositionRef.current === 'infinite-scroll' ? null : keepScrollPositionRef.current;
       return;
     }
     /**
@@ -302,9 +357,15 @@ export const LogsPanel = ({
     }
   }, [panelData.request?.app, isAscending, scrollElement, logRows]);
 
-  const getFieldLinks = useCallback(
-    (field: Field, rowIndex: number) => {
-      return getFieldLinksForExplore({ field, rowIndex, range: panelData.timeRange });
+  const getFieldLinks: GetFieldLinksFn = useCallback(
+    (field, rowIndex, dataFrame, vars) => {
+      return getFieldLinksForExplore({
+        field,
+        rowIndex,
+        range: panelData.timeRange,
+        dataFrame: dataFrame,
+        vars,
+      });
     },
     [panelData]
   );
@@ -370,6 +431,30 @@ export const LogsPanel = ({
     }
   }, [options.displayedFields]);
 
+  // Respect the scroll position when refreshing the panel
+  useEffect(() => {
+    function handleScroll() {
+      if (!scrollElement) {
+        return;
+      }
+      // Signal to keep the user scroll position
+      keepScrollPositionRef.current = 'user';
+      const atTheBottom = scrollElement.scrollHeight - scrollElement.scrollTop - scrollElement.clientHeight === 0;
+      // Except when the user resets the scroll to the original position depending on the sort direction
+      if (scrollElement.scrollTop === 0 && !isAscending) {
+        keepScrollPositionRef.current = null;
+      } else if (atTheBottom && isAscending) {
+        keepScrollPositionRef.current = null;
+      }
+    }
+    scrollElement?.addEventListener('scroll', handleScroll);
+    scrollElement?.addEventListener('wheel', handleScroll);
+    return () => {
+      scrollElement?.removeEventListener('scroll', handleScroll);
+      scrollElement?.removeEventListener('wheel', handleScroll);
+    };
+  }, [isAscending, scrollElement]);
+
   const loadMoreLogs = useCallback(
     async (scrollRange: AbsoluteTimeRange) => {
       if (!data.request || !config.featureToggles.logsInfiniteScrolling || loadingRef.current) {
@@ -391,7 +476,7 @@ export const LogsPanel = ({
         loadingRef.current = false;
       }
 
-      keepScrollPositionRef.current = true;
+      keepScrollPositionRef.current = 'infinite-scroll';
       setPanelData({
         ...panelData,
         series: newSeries,
@@ -400,13 +485,11 @@ export const LogsPanel = ({
     [data.request, dataSourcesMap, onNewLogsReceived, panelData, timeZone]
   );
 
-  if (!data || logRows.length === 0) {
-    return <PanelDataErrorView fieldConfig={fieldConfig} panelId={id} data={data} needsStringField />;
-  }
-
   const renderCommonLabels = () => (
     <div className={cx(style.labelContainer, isAscending && style.labelContainerAscending)}>
-      <span className={style.label}>Common labels:</span>
+      <span className={style.label}>
+        <Trans i18nKey="logs.logs-panel.render-common-labels.common-labels">Common labels:</Trans>
+      </span>
       <LogLabels
         labels={typeof commonLabels?.value === 'object' ? commonLabels?.value : noCommonLabels}
         emptyMessage="(no common labels)"
@@ -414,16 +497,47 @@ export const LogsPanel = ({
     </div>
   );
 
+  const initialScrollPosition = useMemo(() => {
+    /**
+     * In dashboards, users with newest logs at the bottom have the expectation of keeping the scroll at the bottom
+     * when new data is received. See https://github.com/grafana/grafana/pull/37634
+     */
+    if (app === CoreApp.Dashboard || app === CoreApp.PanelEditor) {
+      return sortOrder === LogsSortOrder.Ascending ? 'bottom' : 'top';
+    }
+    return 'top';
+  }, [app, sortOrder]);
+
+  const storageKey = useMemo(() => {
+    if (controlsStorageKey) {
+      return controlsStorageKey;
+    }
+    if (!data.request) {
+      return undefined;
+    }
+    return `${data.request?.dashboardUID}.${id}`;
+  }, [controlsStorageKey, data.request, id]);
+
+  if (!data || logRows.length === 0) {
+    return <PanelDataErrorView fieldConfig={fieldConfig} panelId={id} data={data} needsStringField />;
+  }
+
   // Passing callbacks control the display of the filtering buttons. We want to pass it only if onAddAdHocFilter is defined.
   const defaultOnClickFilterLabel = onAddAdHocFilter ? handleOnClickFilterLabel : undefined;
   const defaultOnClickFilterOutLabel = onAddAdHocFilter ? handleOnClickFilterOutLabel : undefined;
 
   const onClickShowField = isOnClickShowField(options.onClickShowField) ? options.onClickShowField : showField;
   const onClickHideField = isOnClickHideField(options.onClickHideField) ? options.onClickHideField : hideField;
+  const setDisplayedFieldsFn = isSetDisplayedFields(options.setDisplayedFields)
+    ? options.setDisplayedFields
+    : setDisplayedFields;
+
+  // In Dashboards, default to inline. Otherwise, let apps control or have automatic behavior.
+  const detailsMode = detailsModeProp ? detailsModeProp : app === CoreApp.Dashboard ? 'inline' : undefined;
 
   return (
     <>
-      {contextRow && (
+      {(!config.featureToggles.newLogsPanel || !config.featureToggles.newLogContext) && contextRow && (
         <LogRowContextModal
           open={contextRow !== null}
           row={contextRow}
@@ -434,39 +548,47 @@ export const LogsPanel = ({
           getLogRowContextUi={getLogRowContextUi}
         />
       )}
-      <ScrollContainer ref={(scrollElement) => setScrollElement(scrollElement)}>
-        <div onMouseLeave={onLogContainerMouseLeave} className={style.container} ref={logsContainerRef}>
-          {showCommonLabels && !isAscending && renderCommonLabels()}
-          <InfiniteScroll
-            loading={infiniteScrolling}
-            loadMoreLogs={enableInfiniteScrolling ? loadMoreLogs : undefined}
-            range={data.timeRange}
-            timeZone={timeZone}
-            rows={logRows}
-            scrollElement={scrollElement}
-            sortOrder={sortOrder}
-          >
-            <LogRows
-              scrollElement={scrollElement}
-              scrollIntoView={scrollIntoView}
-              permalinkedRowId={getLogsPanelState()?.logs?.id ?? undefined}
-              onPermalinkClick={showPermaLink() ? onPermalinkClick : undefined}
-              logRows={logRows}
-              showContextToggle={showContextToggle}
-              deduplicatedRows={deduplicatedRows}
+      {config.featureToggles.newLogsPanel && config.featureToggles.newLogContext && getLogRowContext && contextRow && (
+        <LogLineContext
+          open={contextRow !== null}
+          log={contextRow}
+          onClose={onCloseContext}
+          getRowContext={(row, options) => getLogRowContext(row, contextRow, options)}
+          getLogRowContextUi={getLogRowContextUi}
+          logOptionsStorageKey={controlsStorageKey}
+          timeZone={timeZone}
+          displayedFields={displayedFields}
+          onClickShowField={showField}
+          onClickHideField={hideField}
+        />
+      )}
+      {config.featureToggles.newLogsPanel && (
+        <div
+          onMouseLeave={onLogContainerMouseLeave}
+          className={style.logListContainer}
+          style={height ? { minHeight: height } : undefined}
+          ref={(element: HTMLDivElement) => setScrollElement(element)}
+        >
+          {deduplicatedRows.length > 0 && scrollElement && (
+            <LogList
+              app={isCoreApp(app) ? app : CoreApp.Dashboard}
+              containerElement={scrollElement}
               dedupStrategy={dedupStrategy}
-              showLabels={showLabels}
-              showTime={showTime}
-              wrapLogMessage={wrapLogMessage}
-              prettifyLogMessage={prettifyLogMessage}
-              timeZone={timeZone}
-              getFieldLinks={getFieldLinks}
-              logsSortOrder={sortOrder}
+              detailsMode={detailsMode}
+              displayedFields={displayedFields}
               enableLogDetails={enableLogDetails}
-              previewLimit={isAscending ? logRows.length : undefined}
-              onLogRowHover={onLogRowHover}
-              app={CoreApp.Dashboard}
-              onOpenContext={onOpenContext}
+              fontSize={fontSize}
+              getFieldLinks={getFieldLinks}
+              isLabelFilterActive={isIsFilterLabelActive(isFilterLabelActive) ? isFilterLabelActive : undefined}
+              initialScrollPosition={initialScrollPosition}
+              loading={infiniteScrolling}
+              logLineMenuCustomItems={
+                isLogLineMenuCustomItems(logLineMenuCustomItems) ? logLineMenuCustomItems : undefined
+              }
+              logs={deduplicatedRows}
+              logSupportsContext={showContextToggle}
+              loadMore={enableInfiniteScrolling ? loadMoreLogs : undefined}
+              noInteractions={noInteractions}
               onClickFilterLabel={
                 isOnClickFilterLabel(onClickFilterLabel) ? onClickFilterLabel : defaultOnClickFilterLabel
               }
@@ -477,19 +599,136 @@ export const LogsPanel = ({
               onClickFilterOutString={
                 isOnClickFilterOutString(onClickFilterOutString) ? onClickFilterOutString : undefined
               }
-              isFilterLabelActive={isIsFilterLabelActive(isFilterLabelActive) ? isFilterLabelActive : undefined}
-              displayedFields={displayedFields}
               onClickShowField={displayedFields !== undefined ? onClickShowField : undefined}
               onClickHideField={displayedFields !== undefined ? onClickHideField : undefined}
-              logRowMenuIconsBefore={isReactNodeArray(logRowMenuIconsBefore) ? logRowMenuIconsBefore : undefined}
-              logRowMenuIconsAfter={isReactNodeArray(logRowMenuIconsAfter) ? logRowMenuIconsAfter : undefined}
-              // Ascending order causes scroll to stick to the bottom, so previewing is futile
-              renderPreview={isAscending ? false : true}
+              onLogLineHover={onLogRowHover}
+              onLogOptionsChange={isOnLogOptionsChange(onLogOptionsChange) ? onLogOptionsChange : undefined}
+              onOpenContext={onOpenContext}
+              onPermalinkClick={showPermaLink() ? onPermalinkClick : undefined}
+              permalinkedLogId={getLogsPanelState()?.logs?.id ?? undefined}
+              setDisplayedFields={setDisplayedFieldsFn}
+              showControls={Boolean(showControls)}
+              showTime={showTime}
+              sortOrder={sortOrder}
+              logOptionsStorageKey={storageKey}
+              syntaxHighlighting={syntaxHighlighting}
+              timeRange={data.timeRange}
+              timestampResolution={timestampResolution}
+              timeZone={timeZone}
+              wrapLogMessage={wrapLogMessage}
             />
-          </InfiniteScroll>
+          )}
+        </div>
+      )}
+      {!config.featureToggles.newLogsPanel && !showControls && (
+        <ScrollContainer ref={(scrollElement) => setScrollElement(scrollElement)}>
+          <div onMouseLeave={onLogContainerMouseLeave} className={style.container} ref={logsContainerRef}>
+            {showCommonLabels && !isAscending && renderCommonLabels()}
+            <InfiniteScroll
+              loading={infiniteScrolling}
+              loadMoreLogs={enableInfiniteScrolling ? loadMoreLogs : undefined}
+              range={data.timeRange}
+              timeZone={timeZone}
+              rows={logRows}
+              scrollElement={scrollElement}
+              sortOrder={sortOrder}
+            >
+              <LogRows
+                scrollElement={scrollElement}
+                scrollIntoView={scrollIntoView}
+                permalinkedRowId={getLogsPanelState()?.logs?.id ?? undefined}
+                onPermalinkClick={showPermaLink() ? onPermalinkClick : undefined}
+                logRows={logRows}
+                showContextToggle={showContextToggle}
+                deduplicatedRows={deduplicatedRows}
+                dedupStrategy={dedupStrategy}
+                showLabels={showLabels}
+                showTime={showTime}
+                wrapLogMessage={wrapLogMessage}
+                prettifyLogMessage={prettifyLogMessage}
+                timeZone={timeZone}
+                getFieldLinks={getFieldLinks}
+                logsSortOrder={sortOrder}
+                enableLogDetails={enableLogDetails}
+                previewLimit={isAscending ? logRows.length : undefined}
+                onLogRowHover={onLogRowHover}
+                app={isCoreApp(app) ? app : CoreApp.Dashboard}
+                onOpenContext={onOpenContext}
+                onClickFilterLabel={
+                  isOnClickFilterLabel(onClickFilterLabel) ? onClickFilterLabel : defaultOnClickFilterLabel
+                }
+                onClickFilterOutLabel={
+                  isOnClickFilterOutLabel(onClickFilterOutLabel) ? onClickFilterOutLabel : defaultOnClickFilterOutLabel
+                }
+                onClickFilterString={isOnClickFilterString(onClickFilterString) ? onClickFilterString : undefined}
+                onClickFilterOutString={
+                  isOnClickFilterOutString(onClickFilterOutString) ? onClickFilterOutString : undefined
+                }
+                isFilterLabelActive={isIsFilterLabelActive(isFilterLabelActive) ? isFilterLabelActive : undefined}
+                displayedFields={displayedFields}
+                onClickShowField={displayedFields !== undefined ? onClickShowField : undefined}
+                onClickHideField={displayedFields !== undefined ? onClickHideField : undefined}
+                logRowMenuIconsBefore={isReactNodeArray(logRowMenuIconsBefore) ? logRowMenuIconsBefore : undefined}
+                logRowMenuIconsAfter={isReactNodeArray(logRowMenuIconsAfter) ? logRowMenuIconsAfter : undefined}
+                // Ascending order causes scroll to stick to the bottom, so previewing is futile
+                renderPreview={isAscending ? false : true}
+              />
+            </InfiniteScroll>
+            {showCommonLabels && isAscending && renderCommonLabels()}
+          </div>
+        </ScrollContainer>
+      )}
+      {!config.featureToggles.newLogsPanel && showControls && (
+        <div onMouseLeave={onLogContainerMouseLeave} className={style.controlledLogsContainer}>
+          {showCommonLabels && !isAscending && renderCommonLabels()}
+          <ControlledLogRows
+            ref={(scrollElement: HTMLDivElement | null) => setScrollElement(scrollElement)}
+            visualisationType="logs"
+            loading={infiniteScrolling}
+            loadMoreLogs={enableInfiniteScrolling ? loadMoreLogs : undefined}
+            range={data.timeRange}
+            logRows={logRows}
+            deduplicatedRows={deduplicatedRows}
+            dedupStrategy={dedupStrategy}
+            onClickFilterLabel={
+              isOnClickFilterLabel(onClickFilterLabel) ? onClickFilterLabel : defaultOnClickFilterLabel
+            }
+            onClickFilterOutLabel={
+              isOnClickFilterOutLabel(onClickFilterOutLabel) ? onClickFilterOutLabel : defaultOnClickFilterOutLabel
+            }
+            showContextToggle={showContextToggle}
+            showLabels={showLabels}
+            showTime={showTime}
+            enableLogDetails={enableLogDetails}
+            wrapLogMessage={wrapLogMessage}
+            prettifyLogMessage={prettifyLogMessage}
+            timeZone={timeZone}
+            getFieldLinks={getFieldLinks}
+            logsSortOrder={sortOrder}
+            displayedFields={displayedFields}
+            onClickShowField={displayedFields !== undefined ? onClickShowField : undefined}
+            onClickHideField={displayedFields !== undefined ? onClickHideField : undefined}
+            app={isCoreApp(app) ? app : CoreApp.Dashboard}
+            onLogRowHover={onLogRowHover}
+            onOpenContext={onOpenContext}
+            onPermalinkClick={onPermalinkClick}
+            permalinkedRowId={getLogsPanelState()?.logs?.id ?? undefined}
+            scrollIntoView={scrollIntoView}
+            isFilterLabelActive={isIsFilterLabelActive(isFilterLabelActive) ? isFilterLabelActive : undefined}
+            onClickFilterString={isOnClickFilterString(onClickFilterString) ? onClickFilterString : undefined}
+            onClickFilterOutString={
+              isOnClickFilterOutString(onClickFilterOutString) ? onClickFilterOutString : undefined
+            }
+            logRowMenuIconsBefore={isReactNodeArray(logRowMenuIconsBefore) ? logRowMenuIconsBefore : undefined}
+            logRowMenuIconsAfter={isReactNodeArray(logRowMenuIconsAfter) ? logRowMenuIconsAfter : undefined}
+            onLogOptionsChange={isOnLogOptionsChange(onLogOptionsChange) ? onLogOptionsChange : undefined}
+            logOptionsStorageKey={controlsStorageKey}
+            // Ascending order causes scroll to stick to the bottom, so previewing is futile
+            renderPreview={isAscending ? false : true}
+          />
           {showCommonLabels && isAscending && renderCommonLabels()}
         </div>
-      </ScrollContainer>
+      )}
     </>
   );
 };
@@ -498,6 +737,17 @@ const getStyles = (theme: GrafanaTheme2) => ({
   container: css({
     marginBottom: theme.spacing(1.5),
   }),
+  logListContainer: css({
+    minHeight: '100%',
+    maxHeight: '100%',
+    display: 'flex',
+    flex: 1,
+    flexDirection: 'column',
+    overflow: 'hidden',
+  }),
+  controlledLogsContainer: css({
+    height: '100%',
+  }),
   labelContainer: css({
     margin: theme.spacing(0, 0, 0.5, 0.5),
     display: 'flex',
@@ -574,7 +824,7 @@ export async function requestMoreLogs(
     return [];
   }
 
-  const range: TimeRange = convertRawToRange({
+  const range: TimeRange = rangeUtil.convertRawToRange({
     from: dateTimeForTimeZone(timeZone, timeRange.from),
     to: dateTimeForTimeZone(timeZone, timeRange.to),
   });
